@@ -31,6 +31,11 @@ use Doctrine\Common\DataFixtures\ReferenceRepository;
  */
 class ORMExecutor extends AbstractExecutor
 {
+    /**
+     * @var EntityManagerInterface
+     */
+    private $em;
+    
     /**
      * Construct new fixtures loader instance.
      *
